/************************************************************************
 *   MODULE  : ELEVATIO.C
 *
 *   PURPOSE : obtains the elevation at the observer's position
 *
 *   AUTHOR  : Ron Hawkins & Craig Woodfolk, SAIC (original)
 *             Kevin Russo, SAIC (1, 3)
 *             Melissa Mathews, SAIC (2)
 *
 *   Change     Date          Reason
 *   ------     ----          ------
 *   Original   25 Sept 92    n/a
 *   1          20 Oct 92     overall program update
 *   2          17 Jun 93     converted xvt 2.0 calls to xvt 3.1
 *   3          02 Nov 93     added routine to interpolate DTED for given
 *                            lat and lon
 *
 ************************************************************************/

/****************
 * header files *
 ****************/

#include "los.h"

/*********************
 * global variables  *
 *********************/

extern RASTER elev_raster;                 /* DTED elevations           */

/************************************************************************
 *   FUNCTION     : obsv_elev
 *
 *   PURPOSE      : obtains the given position's elevation using the
 *                  nearest neighbor approach for speed of response
 *
 *   INPUTS       : lat - the position latitude
 *                  lon - the position longitude
 *
 *   RETURN VALUE : the position elevation (short)
 *
 *   CALLED FROM  : multiple places in the LOS code
 *
 ************************************************************************/

#ifdef PROTO
short obsv_elev (GFLOAT lat, GFLOAT lon)
#else
short obsv_elev (lat, lon)
GFLOAT lat, lon;
#endif

{

   long                 row, col;           /* dted row & column         */
   short                elevation;          /* elevation at row, col     */
   short pHMEM          elev_ptr;           /* pointer to lat, lon site  */

   /* initialize pointer */
   elev_ptr = (short pHMEM) xvt_gmem_lock(elev_raster.data_matrix.ghandle);
   if ( elev_ptr == (short pHMEM) NULL )
      xvt_dm_post_fatal_exit("unable to lock elev_raster in obsv_elev()");

   /* calculate row & column observer location in DTED raster */
   row  = (long) ((lat - elev_raster.data_hdr.extent.bottom) /
                  elev_raster.data_hdr.y_res);
   col  = (long) ((lon - elev_raster.data_hdr.extent.left) /
                  elev_raster.data_hdr.x_res);

   if ((row < ZERO) || (row > elev_raster.data_matrix.height-1) ||
       (col < ZERO) || (col > elev_raster.data_matrix.width-1))
      elevation = UNKNOWN_DTED;
   else
      elevation = elev_ptr[row * elev_raster.data_matrix.width + col];

   xvt_gmem_unlock (elev_raster.data_matrix.ghandle);

   return elevation;
}

/************************************************************************
 *   FUNCTION     : interp_elev
 *
 *   PURPOSE      : interpolates an elevation values based on its
 *                  nearest neighbors
 *
 *   INPUTS       : dted - pointer the dted data in memory
 *                  lat  - the position latitude
 *                  lon  - the position longitude
 *
 *   RETURN VALUE : short - the interpolated DTED value
 *
 *   CALLED FROM  : make_one_radial in module MAKE_RDL.C
 *
 *   CALLS        : none
 *
 ************************************************************************/

#ifdef PROTO
short interp_elev (short pHMEM dted, GFLOAT lat, GFLOAT lon)
#else
short interp_elev (dted, lat, lon)
short pHMEM dted;
GFLOAT lat, lon;
#endif
{
   GFLOAT row, col;                    /* row and col in dted matrix    */
   GFLOAT deltax, deltay, opdx, opdy;  /* x/y dif from row/col          */

   short elev[4], newelev;             /* temp DTED and final result    */
   short minval, maxval;               /* used in dted checks           */
   long  i, j, index;                  /* starting point in interp      */
   long  widm1, hgtm1;

   row = (lat - elev_raster.data_hdr.extent.bottom) /
         elev_raster.data_hdr.y_res;

   col = (lon - elev_raster.data_hdr.extent.left) /
         elev_raster.data_hdr.x_res;

   if ((row < (GFLOAT)ZERO) || (row > (GFLOAT)elev_raster.data_matrix.height-1) ||
       (col < (GFLOAT)ZERO) || (col > (GFLOAT)elev_raster.data_matrix.width-1))
      return UNKNOWN_DTED;

   i = (long)col;
   j = (long)row + 1;

   index = j * elev_raster.data_matrix.width + i;

   deltax = fabs(col - (GFLOAT)i);
   deltay = fabs(row - (GFLOAT)j);

   opdx = 1.0 - deltax;
   opdy = 1.0 - deltay;

   widm1 = elev_raster.data_matrix.width - 1;
   hgtm1 = elev_raster.data_matrix.height - 1;

   if ((i >= (long)ZERO) && (j > (long)ZERO) && (i < widm1) && (j <= hgtm1))
   {
      maxval = (short)-32767;
      minval = (short)32767;

      elev[0] = *(dted + index);
      maxval = maxof(maxval, elev[0]);
      minval = minof(minval, elev[0]);

      elev[1] = *(dted + index + 1);
      maxval = maxof(maxval, elev[1]);
      minval = minof(minval, elev[1]);

      elev[2] = *(dted + index - elev_raster.data_matrix.width);
      maxval = maxof(maxval, elev[2]);
      minval = minof(minval, elev[2]);

      elev[3] = *(dted + index - elev_raster.data_matrix.width + 1);
      maxval = maxof(maxval, elev[3]);
      minval = minof(minval, elev[3]);

      if ((elev[0] < (short)ZERO) || (elev[1] < (short)ZERO) ||
          (elev[2] < (short)ZERO) || (elev[3] < (short)ZERO))
      {
         newelev = UNKNOWN_DTED;
      }
      else
      {
         if (minval == maxval)
            newelev = minval;
         else
            newelev = (short)(opdx * opdy * (GFLOAT)elev[0] +
                      opdx * deltay * (GFLOAT)elev[2] +
                      opdy * deltax * (GFLOAT)elev[1] +
                      deltax * deltay * (GFLOAT)elev[3]);

         if (newelev < minval)
            newelev = minval;
         else if (newelev > maxval)
            newelev = maxval;
      }
   }
   else
   {
      if (i < (long)ZERO)
         i = (long)ZERO;
      if (i > widm1)
         i = widm1;
      if (j < (long)ZERO)
         j = (long)ZERO;
      if (j > hgtm1)
         j = hgtm1;

      newelev = dted[j * elev_raster.data_matrix.width + i];

   }

   return newelev;
}
