/************************************************************************
 *   MODULE  : PRECISE.C
 *
 *   PURPOSE : provides routines to get the 5x5 array of DTED around the
 *             observer and calculate a new position when one of the
 *             DTED positions is chosen.  Both the precise_array and
 *             precise_position functions are called by do_dlg101 in module
 *             LOSD101.C
 *
 *   AUTHOR  : Ron Hawkins, SAIC (original)
 *             Kevin Russo, SAIC (1)
 *
 *   Change     Date          Reason
 *   ------     ----          ------
 *   Original   25 Sep 92     n/a
 *   1          21 Jun 93     update to XVT 3.0 and MUSE Beta Release 3
 *
 ************************************************************************/

/****************
 * header files *
 ****************/

#include "los.h"

 /* precise_array accepts an observer position and returns
  * a 5 x 5 array of elevation values centered around the
  * observer posit.  These values are used in the user
  * interface to let the user position his observer at an
  * exact elev. post value.
  *
  * NOTE: the calling routine is responsible for freeing
  * up the memory allocated for precise_matrix.
  */

/************************
 * file scope variables *
 ************************/

static short srow, scol;                  /* last selected post         */

static GFLOAT slat = ERRCOORD;            /* last latitude              */
static GFLOAT slon = ERRCOORD;            /* last longitude             */

static short  obsv_row;                   /* starting observer row      */
static short  obsv_col;                   /* staring observer column    */

/************************************************************************
 *   FUNCTION     : precise_array
 *
 *   PURPOSE      : precise_array accepts an observer position and returns
 *                  a 5 x 5 array of elevation values centered around the
 *                  observer position.  These values are used in the user
 *                  interface to let the user position his observer at an
 *                  exact elevation position value.
 *
 *                  Note : the calling routine is responsible for freeing
 *                         up the memory allocated for the precise matrix.
 *
 *   INPUTS       : obsv_lat       - observer latitude in dec. degrees
 *                  obsv_lon       - observer longitude in dec. degrees
 *                  elev_raster    - pointer to the DTED raster
 *                  precise_matrix - pointer to the 5 x 5 matrix.
 *
 *   RETURN VALUE : -1 on error 
 *
 *   CALLED FROM  : do_dted_dlg in module LOSD101.C
 *
 *   CALLS        : none
 *
 ************************************************************************/

#ifdef PROTO
int precise_array ( GFLOAT obsv_lat, GFLOAT obsv_lon,
                RASTER *elev_raster, GHANDLE
                *precise_matrix )
#else
int precise_array ( obsv_lat, obsv_lon, elev_raster,
                 precise_matrix )
GFLOAT obsv_lat, obsv_lon;
RASTER *elev_raster;
GHANDLE *precise_matrix;
#endif
{
    GFLOAT temp_row, temp_col, lat, lon;

    char buf[25];

    short row_delta, col_delta, index;
    short curr_col, curr_row;
    short pHMEM dted_ptr;
    short *temp_ptr;

    /* calc. row & col of observer location in the DTED */
    sprintf(buf, "%.4lf", slat);
    slat = (GFLOAT)atof(buf);
    sprintf(buf, "%.4lf", slon);
    slon = (GFLOAT)atof(buf);

    sprintf(buf, "%.4lf", obsv_lat);
    lat = (GFLOAT)atof(buf);
    sprintf(buf, "%.4lf", obsv_lon);
    lon = (GFLOAT)atof(buf);

    if ((lat == slat) && (lon == slon))
    {
       obsv_row = srow;
       obsv_col = scol;
    }
    else
    {
       temp_row = ((obsv_lat - elev_raster->data_hdr.extent.bottom) /
                    elev_raster->data_hdr.y_res);
       temp_col = ((obsv_lon - elev_raster->data_hdr.extent.left) /
                    elev_raster->data_hdr.x_res);

       temp_row += (GFLOAT)0.5;
       temp_col += (GFLOAT)0.5;

       obsv_row = (short)temp_row;
       obsv_col = (short)temp_col;
    }

    /* alloc memory for a 5 x 5 matrix of elev. vals */
    *precise_matrix = xvt_gmem_alloc ( 25 * sizeof(short) );
    if (!*precise_matrix)
    {
      /*       xvt_dm_post_error("Unable to alloc memory for precise positioning matrix");*/
       return -1;
    }

    /* NOTE: if for some reason, memory for the precise
     * matrix cannot be allocated, the program should go
     * ahead using the original observer location.
     */

    dted_ptr = (short pHMEM) xvt_gmem_lock (elev_raster->data_matrix.ghandle);
    if (dted_ptr == NULL)
       xvt_dm_post_fatal_exit("unable to lock dted data in function precise_array");

    temp_ptr = (short *) xvt_gmem_lock (*precise_matrix);
    if (temp_ptr == NULL)
       xvt_dm_post_fatal_exit("unable to lock precise matrix in function precise_array");

    /* cycle through the row & col values, filling the
     * precise matrix.
     */
    for (row_delta = -2, index = 0; row_delta < 3; row_delta++)
    {
       for (col_delta = -2; col_delta < 3; col_delta++)
       {
          curr_row = obsv_row + row_delta;
          curr_col = obsv_col + col_delta;

          /* index into dted data */
          if ((curr_row < ZERO) || (curr_row > elev_raster->data_matrix.height-1) ||
              (curr_col < ZERO) || (curr_col > elev_raster->data_matrix.width-1))
             temp_ptr[index] = UNKNOWN_DTED;
          else
             temp_ptr[index] = dted_ptr[ (long) curr_row *
                               elev_raster->data_matrix.width +
                               (long) curr_col];
          index++;
       }
    }

    xvt_gmem_unlock (elev_raster->data_matrix.ghandle);
    xvt_gmem_unlock (*precise_matrix);
    return 0;
 }

/************************************************************************
 *   FUNCTION     : precise_position
 *
 *   PURPOSE      : finds a new observer location based on a chosen
 *                  row and column within the 5 x 5 DTED matrix.
 *
 *   INPUTS       : selected_row - the newly selectd row in the matrix
 *                  selected_col - the newly selected col in the matrix
 *                  dted_raster  - pointer to the DTED raster
 *                  obsv_lat     - orig. observer latitude in dec. degrees
 *                  obsv_lon     - orig. observer longitude in dec. degrees
 *                  new_lat      - new observer latitude in dec. degrees
 *                  new_lon      - new observer longitude in dec. degrees
 *
 *   RETURN VALUE : void 
 *
 *   CALLED FROM  : do_dted_dlg in module LOSD101.C
 *
 *   CALLS        : none
 *
 ************************************************************************/

#ifdef PROTO
void precise_position (short selected_row, short selected_col,
                   RASTER *dted_raster,
                   GFLOAT obsv_lat, GFLOAT obsv_lon,
                   GFLOAT *new_lat, GFLOAT *new_lon)
#else
void precise_position (selected_row, selected_col, dted_raster,
                   obsv_lat, obsv_lon, new_lat, new_lon)
short selected_row, selected_col;
RASTER *dted_raster;
GFLOAT obsv_lat, obsv_lon, *new_lat, *new_lon;
#endif
{
    short new_row, new_col;

    NOREF (obsv_lat);
    NOREF (obsv_lon);

    /* calc the new row & col from the user selection */
    new_row = srow = obsv_row + (selected_row - 2);
    new_col = scol = obsv_col + (selected_col - 2);

    /* calc the new obsv lat & lon */
    *new_lon = slon = dted_raster->data_hdr.extent.left + ((GFLOAT) new_col *
                              dted_raster->data_hdr.x_res);
    *new_lat = slat = dted_raster->data_hdr.extent.bottom + ((GFLOAT) new_row *
                              dted_raster->data_hdr.y_res);
 }
